@@ -1,7 +1,8 @@
 from sympy.core import symbols, S
-from sympy.matrices.expressions import MatrixSymbol, Inverse
-from sympy.matrices import eye, Identity, ShapeError
-from sympy.utilities.pytest import raises
+from sympy.matrices.expressions import MatrixSymbol, Inverse, MatPow, ZeroMatrix, OneMatrix
+from sympy.matrices.common import NonSquareMatrixError, NonInvertibleMatrixError
+from sympy.matrices import eye, Identity
+from sympy.testing.pytest import raises
 from sympy import refine, Q
 
 n, m, l = symbols('n m l', integer=True)
@@ -13,14 +14,12 @@ E = MatrixSymbol('E', m, n)
 
 
 def test_inverse():
-    raises(ShapeError, lambda: Inverse(A))
-    raises(ShapeError, lambda: Inverse(A*B))
-
     assert Inverse(C).args == (C, S.NegativeOne)
     assert Inverse(C).shape == (n, n)
     assert Inverse(A*E).shape == (n, n)
     assert Inverse(E*A).shape == (m, m)
     assert Inverse(C).inverse() == C
+    assert Inverse(Inverse(C)).doit() == C
     assert isinstance(Inverse(Inverse(C)), Inverse)
 
     assert Inverse(*Inverse(E*A).args) == Inverse(E*A)
@@ -41,6 +40,21 @@ def test_inverse():
     assert Inverse(eye(3)).doit() == eye(3)
     assert Inverse(eye(3)).doit(deep=False) == eye(3)
 
+    assert OneMatrix(1, 1).I == Identity(1)
+    assert isinstance(OneMatrix(n, n).I, Inverse)
+
+def test_inverse_non_invertible():
+    raises(NonSquareMatrixError, lambda: Inverse(A))
+    raises(NonSquareMatrixError, lambda: Inverse(A*B))
+    raises(NonSquareMatrixError, lambda: ZeroMatrix(n, m).I)
+    raises(NonInvertibleMatrixError, lambda: ZeroMatrix(n, n).I)
+    raises(NonSquareMatrixError, lambda: OneMatrix(n, m).I)
+    raises(NonInvertibleMatrixError, lambda: OneMatrix(2, 2).I)
 
 def test_refine():
     assert refine(C.I, Q.orthogonal(C)) == C.T
+
+
+def test_inverse_matpow_canonicalization():
+    A = MatrixSymbol('A', 3, 3)
+    assert Inverse(MatPow(A, 3)).doit() == MatPow(Inverse(A), 3).doit()
