@@ -18,8 +18,11 @@ for:
 """
 
 from sympy import Derivative, S
-from sympy.core.compatibility import iterable, range
+from sympy.core.basic import preorder_traversal
+from sympy.core.compatibility import iterable
 from sympy.core.decorators import deprecated
+from sympy.core.function import Subs
+from sympy.utilities.exceptions import SymPyDeprecationWarning
 
 
 def finite_diff_weights(order, x_list, x0=S.One):
@@ -29,7 +32,6 @@ def finite_diff_weights(order, x_list, x0=S.One):
     0, 1, ..., up to ``order`` using a recursive formula. Order of accuracy
     is at least ``len(x_list) - order``, if ``x_list`` is defined correctly.
 
-
     Parameters
     ==========
 
@@ -123,7 +125,6 @@ def finite_diff_weights(order, x_list, x0=S.One):
     Note that, unless you plan on using approximations based on subsets of
     ``x_list``, the order of gridpoints does not matter.
 
-
     The capability to generate weights at arbitrary points can be
     used e.g. to minimize Runge's phenomenon by using Chebyshev nodes:
 
@@ -137,7 +138,7 @@ def finite_diff_weights(order, x_list, x0=S.One):
     >>> [simplify(c) for c in  mycoeffs] #doctest: +NORMALIZE_WHITESPACE
     [(h**3/2 + h**2*x - 3*h*x**2 - 4*x**3)/h**4,
     (-sqrt(2)*h**3 - 4*h**2*x + 3*sqrt(2)*h*x**2 + 8*x**3)/h**4,
-    6*x/h**2 - 8*x**3/h**4,
+    (6*h**2*x - 8*x**3)/h**4,
     (sqrt(2)*h**3 - 4*h**2*x - 3*sqrt(2)*h*x**2 + 8*x**3)/h**4,
     (-h**3/2 + h**2*x + 3*h*x**2 - 4*x**3)/h**4]
 
@@ -157,7 +158,6 @@ def finite_diff_weights(order, x_list, x0=S.One):
 
     sympy.calculus.finite_diff.apply_finite_diff
 
-
     References
     ==========
 
@@ -243,9 +243,9 @@ def apply_finite_diff(order, x_list, y_list, x0=S.Zero):
     >>> i = Idx('i')
     >>> x_list, y_list = zip(*[(x[i+j], y[i+j]) for j in range(-1,2)])
     >>> apply_finite_diff(1, x_list, y_list, x[i])
-    ((x[i + 1] - x[i])/(-x[i - 1] + x[i]) - 1)*y[i]/(x[i + 1] - x[i]) - \
-(x[i + 1] - x[i])*y[i - 1]/((x[i + 1] - x[i - 1])*(-x[i - 1] + x[i])) + \
-(-x[i - 1] + x[i])*y[i + 1]/((x[i + 1] - x[i - 1])*(x[i + 1] - x[i]))
+    ((x[i + 1] - x[i])/(-x[i - 1] + x[i]) - 1)*y[i]/(x[i + 1] - x[i]) -
+    (x[i + 1] - x[i])*y[i - 1]/((x[i + 1] - x[i - 1])*(-x[i - 1] + x[i])) +
+    (-x[i - 1] + x[i])*y[i + 1]/((x[i + 1] - x[i - 1])*(x[i + 1] - x[i]))
 
     Notes
     =====
@@ -315,7 +315,6 @@ def _as_finite_diff(derivative, points=1, x0=None, wrt=None):
         derivative is to be approximated for. If not provided it
         is required that the Derivative is ordinary. Default: ``None``.
 
-
     Examples
     ========
 
@@ -347,10 +346,9 @@ def _as_finite_diff(derivative, points=1, x0=None, wrt=None):
     >>> e, sq2 = exp(1), sqrt(2)
     >>> xl = [x-h, x+h, x+e*h]
     >>> as_finite_diff(f(x).diff(x, 1), xl, x+h*sq2)
-    2*h*((h + sqrt(2)*h)/(2*h) - (-sqrt(2)*h + h)/(2*h))*f(E*h + x)/\
-((-h + E*h)*(h + E*h)) + (-(-sqrt(2)*h + h)/(2*h) - \
-(-sqrt(2)*h + E*h)/(2*h))*f(-h + x)/(h + E*h) + \
-(-(h + sqrt(2)*h)/(2*h) + (-sqrt(2)*h + E*h)/(2*h))*f(h + x)/(-h + E*h)
+    2*h*((h + sqrt(2)*h)/(2*h) - (-sqrt(2)*h + h)/(2*h))*f(E*h + x)/((-h + E*h)*(h + E*h)) +
+    (-(-sqrt(2)*h + h)/(2*h) - (-sqrt(2)*h + E*h)/(2*h))*f(-h + x)/(h + E*h) +
+    (-(h + sqrt(2)*h)/(2*h) + (-sqrt(2)*h + E*h)/(2*h))*f(h + x)/(-h + E*h)
 
     Partial derivatives are also supported:
 
@@ -424,60 +422,65 @@ as_finite_diff.__doc__ = """
 
 
 def differentiate_finite(expr, *symbols,
-                         # points=1, x0=None, wrt=None, evaluate=True, #Py2:
-                         **kwargs):
+                         points=1, x0=None, wrt=None, evaluate=False):
     r""" Differentiate expr and replace Derivatives with finite differences.
 
     Parameters
     ==========
+
     expr : expression
     \*symbols : differentiate with respect to symbols
-    points: sequence or coefficient, optional
+    points: sequence, coefficient or undefined function, optional
         see ``Derivative.as_finite_difference``
     x0: number or Symbol, optional
         see ``Derivative.as_finite_difference``
     wrt: Symbol, optional
         see ``Derivative.as_finite_difference``
-    evaluate : bool
-        kwarg passed on to ``diff``, whether or not to
-        evaluate the Derivative intermediately (default: ``False``).
-
 
     Examples
     ========
 
-    >>> from sympy import cos, sin, Function, differentiate_finite
+    >>> from sympy import sin, Function, differentiate_finite
     >>> from sympy.abc import x, y, h
     >>> f, g = Function('f'), Function('g')
     >>> differentiate_finite(f(x)*g(x), x, points=[x-h, x+h])
     -f(-h + x)*g(-h + x)/(2*h) + f(h + x)*g(h + x)/(2*h)
 
-    Note that the above form preserves the product rule in discrete form.
-    If we want we can pass ``evaluate=True`` to get another form (which is
-    usually not what we want):
-
-    >>> differentiate_finite(f(x)*g(x), x, points=[x-h, x+h], evaluate=True).simplify()
-    -((f(-h + x) - f(h + x))*g(x) + (g(-h + x) - g(h + x))*f(x))/(2*h)
-
-    ``differentiate_finite`` works on any expression:
+    ``differentiate_finite`` works on any expression, including the expressions
+    with embedded derivatives:
 
     >>> differentiate_finite(f(x) + sin(x), x, 2)
     -2*f(x) + f(x - 1) + f(x + 1) - 2*sin(x) + sin(x - 1) + sin(x + 1)
-    >>> differentiate_finite(f(x) + sin(x), x, 2, evaluate=True)
-    -2*f(x) + f(x - 1) + f(x + 1) - sin(x)
     >>> differentiate_finite(f(x, y), x, y)
     f(x - 1/2, y - 1/2) - f(x - 1/2, y + 1/2) - f(x + 1/2, y - 1/2) + f(x + 1/2, y + 1/2)
+    >>> differentiate_finite(f(x)*g(x).diff(x), x)
+    (-g(x) + g(x + 1))*f(x + 1/2) - (g(x) - g(x - 1))*f(x - 1/2)
+
+    To make finite difference with non-constant discretization step use
+    undefined functions:
+
+    >>> dx = Function('dx')
+    >>> differentiate_finite(f(x)*g(x).diff(x), points=dx(x))
+    -(-g(x - dx(x)/2 - dx(x - dx(x)/2)/2)/dx(x - dx(x)/2) +
+    g(x - dx(x)/2 + dx(x - dx(x)/2)/2)/dx(x - dx(x)/2))*f(x - dx(x)/2)/dx(x) +
+    (-g(x + dx(x)/2 - dx(x + dx(x)/2)/2)/dx(x + dx(x)/2) +
+    g(x + dx(x)/2 + dx(x + dx(x)/2)/2)/dx(x + dx(x)/2))*f(x + dx(x)/2)/dx(x)
 
     """
-    # Key-word only arguments only available in Python 3
-    points = kwargs.pop('points', 1)
-    x0 = kwargs.pop('x0', None)
-    wrt = kwargs.pop('wrt', None)
-    evaluate = kwargs.pop('evaluate', False)
-    if kwargs:
-        raise ValueError("Unknown kwargs: %s" % kwargs)
+    if any(term.is_Derivative for term in list(preorder_traversal(expr))):
+        evaluate = False
 
     Dexpr = expr.diff(*symbols, evaluate=evaluate)
-    return Dexpr.replace(
-        lambda arg: arg.is_Derivative,
-        lambda arg: arg.as_finite_difference(points=points, x0=x0, wrt=wrt))
+    if evaluate:
+        SymPyDeprecationWarning(feature="``evaluate`` flag",
+                                issue=17881,
+                                deprecated_since_version="1.5").warn()
+        return Dexpr.replace(
+            lambda arg: arg.is_Derivative,
+            lambda arg: arg.as_finite_difference(points=points, x0=x0, wrt=wrt))
+    else:
+        DFexpr = Dexpr.as_finite_difference(points=points, x0=x0, wrt=wrt)
+        return DFexpr.replace(
+            lambda arg: isinstance(arg, Subs),
+            lambda arg: arg.expr.as_finite_difference(
+                    points=points, x0=arg.point[0], wrt=arg.variables[0]))
