@@ -31,7 +31,6 @@ import re
 import os
 import inspect
 from sympy import sympify, Function, Set, Symbol
-from sympy.core.compatibility import string_types
 from sympy.printing import StrPrinter
 from sympy.utilities.misc import debug
 
@@ -250,7 +249,7 @@ def parse_freeq(l, x, cons_index, cons_dict, cons_import, symbols=None):
     res = []
     cons = ''
     for i in l:
-        if isinstance(i, string_types):
+        if isinstance(i, str):
             r = '        return FreeQ({}, {})'.format(i, x)
             # First it checks if a constraint is already present in `cons_dict`, If yes, use it else create a new one.
             if r not in cons_dict.values():
@@ -689,7 +688,7 @@ def rubi_rule_parser(fullform, header=None, module_name='rubi_object'):
 
     if header is None:  # use default header values
         path_header = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
-        header = open(os.path.join(path_header, "header.py.txt"), "r").read()
+        header = open(os.path.join(path_header, "header.py.txt")).read()
         header = header.format(module_name)
 
     cons_dict = {}  # dict keeps track of constraints that has been encountered, thus avoids repetition of constraints.
