@@ -12,10 +12,13 @@ question of adding time to length has no meaning.
 
 from __future__ import division
 
+from typing import Dict as tDict
+
 import collections
 
-from sympy import Integer, Matrix, S, Symbol, sympify, Basic, Tuple, Dict, default_sort_key
-from sympy.core.compatibility import reduce, string_types
+from sympy import (Integer, Matrix, S, Symbol, sympify, Basic, Tuple, Dict,
+    default_sort_key)
+from sympy.core.compatibility import reduce
 from sympy.core.expr import Expr
 from sympy.core.power import Pow
 from sympy.utilities.exceptions import SymPyDeprecationWarning
@@ -23,9 +26,9 @@ from sympy.utilities.exceptions import SymPyDeprecationWarning
 
 class _QuantityMapper(object):
 
-    _quantity_scale_factors_global = {}
-    _quantity_dimensional_equivalence_map_global = {}
-    _quantity_dimension_global = {}
+    _quantity_scale_factors_global = {}  # type: tDict[Expr, Expr]
+    _quantity_dimensional_equivalence_map_global = {}  # type: tDict[Expr, Expr]
+    _quantity_dimension_global = {}  # type: tDict[Expr, Expr]
 
     def __init__(self, *args, **kwargs):
         self._quantity_dimension_map = {}
@@ -132,7 +135,8 @@ class Dimension(Expr):
 
     _op_priority = 13.0
 
-    _dimensional_dependencies = dict()
+    # XXX: This doesn't seem to be used anywhere...
+    _dimensional_dependencies = dict()  # type: ignore
 
     is_commutative = True
     is_number = False
@@ -142,7 +146,7 @@ class Dimension(Expr):
 
     def __new__(cls, name, symbol=None):
 
-        if isinstance(name, string_types):
+        if isinstance(name, str):
             name = Symbol(name)
         else:
             name = sympify(name)
@@ -150,7 +154,7 @@ class Dimension(Expr):
         if not isinstance(name, Expr):
             raise TypeError("Dimension name needs to be a valid math expression")
 
-        if isinstance(symbol, string_types):
+        if isinstance(symbol, str):
             symbol = Symbol(symbol)
         elif symbol is not None:
             assert isinstance(symbol, Symbol)
@@ -241,15 +245,12 @@ class Dimension(Expr):
     def __rmul__(self, other):
         return self.__mul__(other)
 
-    def __div__(self, other):
+    def __truediv__(self, other):
         return self*Pow(other, -1)
 
-    def __rdiv__(self, other):
+    def __rtruediv__(self, other):
         return other * pow(self, -1)
 
-    __truediv__ = __div__
-    __rtruediv__ = __rdiv__
-
     @classmethod
     def _from_dimensional_dependencies(cls, dependencies):
         return reduce(lambda x, y: x * y, (
@@ -258,6 +259,7 @@ class Dimension(Expr):
 
     @classmethod
     def _get_dimensional_dependencies_for_name(cls, name):
+        from sympy.physics.units.systems.si import dimsys_default
         SymPyDeprecationWarning(
             deprecated_since_version="1.2",
             issue=13336,
@@ -267,7 +269,7 @@ class Dimension(Expr):
         return dimsys_default.get_dimensional_dependencies(name)
 
     @property
-    def is_dimensionless(self, dimensional_dependencies=None):
+    def is_dimensionless(self):
         """
         Check if the dimension object really has a dimension.
 
@@ -275,13 +277,15 @@ class Dimension(Expr):
         """
         if self.name == 1:
             return True
-        if dimensional_dependencies is None:
-            SymPyDeprecationWarning(
-                deprecated_since_version="1.2",
-                issue=13336,
-                feature="wrong class",
-            ).warn()
-            dimensional_dependencies=dimsys_default
+
+        from sympy.physics.units.systems.si import dimsys_default
+        SymPyDeprecationWarning(
+            deprecated_since_version="1.2",
+            issue=13336,
+            feature="wrong class",
+        ).warn()
+        dimensional_dependencies=dimsys_default
+
         return dimensional_dependencies.get_dimensional_dependencies(self) == {}
 
     def has_integer_powers(self, dim_sys):
@@ -332,7 +336,7 @@ class DimensionSystem(Basic, _QuantityMapper):
             ).warn()
 
         def parse_dim(dim):
-            if isinstance(dim, string_types):
+            if isinstance(dim, str):
                 dim = Dimension(Symbol(dim))
             elif isinstance(dim, Dimension):
                 pass
@@ -356,7 +360,7 @@ class DimensionSystem(Basic, _QuantityMapper):
         def parse_dim_name(dim):
             if isinstance(dim, Dimension):
                 return dim.name
-            elif isinstance(dim, string_types):
+            elif isinstance(dim, str):
                 return Symbol(dim)
             elif isinstance(dim, Symbol):
                 return dim
@@ -380,7 +384,7 @@ class DimensionSystem(Basic, _QuantityMapper):
                 raise ValueError("Dimension %s both in base and derived" % dim)
             if dim.name not in dimensional_dependencies:
                 # TODO: should this raise a warning?
-                dimensional_dependencies[dim] = Dict({dim.name: 1})
+                dimensional_dependencies[dim.name] = Dict({dim.name: 1})
 
         base_dims.sort(key=default_sort_key)
         derived_dims.sort(key=default_sort_key)
@@ -404,6 +408,11 @@ class DimensionSystem(Basic, _QuantityMapper):
         return self.args[2]
 
     def _get_dimensional_dependencies_for_name(self, name):
+        if isinstance(name, Dimension):
+            name = name.name
+
+        if isinstance(name, str):
+            name = Symbol(name)
 
         if name.is_Symbol:
             # Dimensions not included in the dependencies are considered
@@ -423,6 +432,12 @@ class DimensionSystem(Basic, _QuantityMapper):
                     ret[k] += v
             return {k: v for (k, v) in ret.items() if v != 0}
 
+        if name.is_Add:
+            dicts = [get_for_name(i) for i in name.args]
+            if all([d == dicts[0] for d in dicts[1:]]):
+                return dicts[0]
+            raise TypeError("Only equivalent dimensions can be added or subtracted.")
+
         if name.is_Pow:
             dim = get_for_name(name.base)
             return {k: v*name.exp for (k, v) in dim.items()}
@@ -439,12 +454,9 @@ class DimensionSystem(Basic, _QuantityMapper):
             else:
                 return get_for_name(result)
 
-    def get_dimensional_dependencies(self, name, mark_dimensionless=False):
-        if isinstance(name, Dimension):
-            name = name.name
-        if isinstance(name, string_types):
-            name = Symbol(name)
+        raise TypeError("Type {0} not implemented for get_dimensional_dependencies".format(type(name)))
 
+    def get_dimensional_dependencies(self, name, mark_dimensionless=False):
         dimdep = self._get_dimensional_dependencies_for_name(name)
         if mark_dimensionless and dimdep == {}:
             return {'dimensionless': 1}
