@@ -1,7 +1,7 @@
 """Tools for polynomial factorization routines in characteristic zero. """
 
 from sympy.polys.rings import ring, xring
-from sympy.polys.domains import FF, ZZ, QQ, RR, EX
+from sympy.polys.domains import FF, ZZ, QQ, ZZ_I, QQ_I, RR, EX
 
 from sympy.polys import polyconfig as config
 from sympy.polys.polyerrors import DomainError
@@ -9,9 +9,8 @@ from sympy.polys.polyclasses import ANP
 from sympy.polys.specialpolys import f_polys, w_polys
 
 from sympy import nextprime, sin, sqrt, I
-from sympy.utilities.pytest import raises, XFAIL
+from sympy.testing.pytest import raises, XFAIL
 
-from sympy.core.compatibility import range
 
 f_0, f_1, f_2, f_3, f_4, f_5, f_6 = f_polys()
 w_1, w_2 = w_polys()
@@ -28,7 +27,8 @@ def test_dmp_trial_division():
 
 def test_dup_zz_mignotte_bound():
     R, x = ring("x", ZZ)
-    assert R.dup_zz_mignotte_bound(2*x**2 + 3*x + 4) == 32
+    assert R.dup_zz_mignotte_bound(2*x**2 + 3*x + 4) == 6
+    assert R.dup_zz_mignotte_bound(x**3 + 14*x**2 + 56*x + 64) == 152
 
 
 def test_dmp_zz_mignotte_bound():
@@ -403,6 +403,90 @@ def test_dmp_zz_factor():
                (x**4 + 6*x**2*y + y**2, 1)])
 
 
+def test_dup_qq_i_factor():
+    R, x = ring("x", QQ_I)
+    i = QQ_I(0, 1)
+
+    assert R.dup_qq_i_factor(x**2 - 2) == (QQ_I(1, 0), [(x**2 - 2, 1)])
+
+    assert R.dup_qq_i_factor(x**2 - 1) == (QQ_I(1, 0), [(x - 1, 1), (x + 1, 1)])
+
+    assert R.dup_qq_i_factor(x**2 + 1) == (QQ_I(1, 0), [(x - i, 1), (x + i, 1)])
+
+    assert R.dup_qq_i_factor(x**2/4 + 1) == \
+            (QQ_I(QQ(1, 4), 0), [(x - 2*i, 1), (x + 2*i, 1)])
+
+    assert R.dup_qq_i_factor(x**2 + 4) == \
+            (QQ_I(1, 0), [(x - 2*i, 1), (x + 2*i, 1)])
+
+    assert R.dup_qq_i_factor(x**2 + 2*x + 1) == \
+            (QQ_I(1, 0), [(x + 1, 2)])
+
+    assert R.dup_qq_i_factor(x**2 + 2*i*x - 1) == \
+            (QQ_I(1, 0), [(x + i, 2)])
+
+    f = 8192*x**2 + x*(22656 + 175232*i) - 921416 + 242313*i
+
+    assert R.dup_qq_i_factor(f) == \
+            (QQ_I(8192, 0), [(x + QQ_I(QQ(177, 128), QQ(1369, 128)), 2)])
+
+
+def test_dmp_qq_i_factor():
+    R, x, y = ring("x, y", QQ_I)
+    i = QQ_I(0, 1)
+
+    assert R.dmp_qq_i_factor(x**2 + 2*y**2) == \
+            (QQ_I(1, 0), [(x**2 + 2*y**2, 1)])
+
+    assert R.dmp_qq_i_factor(x**2 + y**2) == \
+            (QQ_I(1, 0), [(x - i*y, 1), (x + i*y, 1)])
+
+    assert R.dmp_qq_i_factor(x**2 + y**2/4) == \
+            (QQ_I(1, 0), [(x - i*y/2, 1), (x + i*y/2, 1)])
+
+    assert R.dmp_qq_i_factor(4*x**2 + y**2) == \
+            (QQ_I(4, 0), [(x - i*y/2, 1), (x + i*y/2, 1)])
+
+
+def test_dup_zz_i_factor():
+    R, x = ring("x", ZZ_I)
+    i = ZZ_I(0, 1)
+
+    assert R.dup_zz_i_factor(x**2 - 2) == (ZZ_I(1, 0), [(x**2 - 2, 1)])
+
+    assert R.dup_zz_i_factor(x**2 - 1) == (ZZ_I(1, 0), [(x - 1, 1), (x + 1, 1)])
+
+    assert R.dup_zz_i_factor(x**2 + 1) == (ZZ_I(1, 0), [(x - i, 1), (x + i, 1)])
+
+    assert R.dup_zz_i_factor(x**2 + 4) == \
+            (ZZ_I(1, 0), [(x - 2*i, 1), (x + 2*i, 1)])
+
+    assert R.dup_zz_i_factor(x**2 + 2*x + 1) == \
+            (ZZ_I(1, 0), [(x + 1, 2)])
+
+    assert R.dup_zz_i_factor(x**2 + 2*i*x - 1) == \
+            (ZZ_I(1, 0), [(x + i, 2)])
+
+    f = 8192*x**2 + x*(22656 + 175232*i) - 921416 + 242313*i
+
+    assert R.dup_zz_i_factor(f) == \
+            (ZZ_I(0, 1), [((64 - 64*i)*x + (773 + 596*i), 2)])
+
+
+def test_dmp_zz_i_factor():
+    R, x, y = ring("x, y", ZZ_I)
+    i = ZZ_I(0, 1)
+
+    assert R.dmp_zz_i_factor(x**2 + 2*y**2) == \
+            (ZZ_I(1, 0), [(x**2 + 2*y**2, 1)])
+
+    assert R.dmp_zz_i_factor(x**2 + y**2) == \
+            (ZZ_I(1, 0), [(x - i*y, 1), (x + i*y, 1)])
+
+    assert R.dmp_zz_i_factor(4*x**2 + y**2) == \
+            (ZZ_I(1, 0), [(2*x - i*y, 1), (2*x + i*y, 1)])
+
+
 def test_dup_ext_factor():
     R, x = ring("x", QQ.algebraic_field(I))
     def anp(element):
