@@ -14,24 +14,15 @@ Hypergeometric
 Rademacher
 """
 
-from __future__ import print_function, division
-
-import random
 
 from sympy import (S, sympify, Rational, binomial, cacheit, Integer,
                    Dummy, Eq, Intersection, Interval,
                    Symbol, Lambda, Piecewise, Or, Gt, Lt, Ge, Le, Contains)
 from sympy import beta as beta_fn
-from sympy.external import import_module
-from sympy.core.compatibility import range
-from sympy.tensor.array import ArrayComprehensionMap
 from sympy.stats.frv import (SingleFiniteDistribution,
                              SingleFinitePSpace)
-from sympy.stats.rv import _value_check, Density, RandomSymbol
+from sympy.stats.rv import _value_check, Density, is_random
 
-numpy = import_module('numpy')
-scipy = import_module('scipy')
-pymc3 = import_module('pymc3')
 
 __all__ = ['FiniteRV',
 'DiscreteUniform',
@@ -44,11 +35,16 @@ __all__ = ['FiniteRV',
 'Rademacher'
 ]
 
-def rv(name, cls, *args):
+def rv(name, cls, *args, **kwargs):
     args = list(map(sympify, args))
     dist = cls(*args)
-    dist.check(*args)
-    return SingleFinitePSpace(name, dist).value
+    if kwargs.pop('check', True):
+        dist.check(*args)
+    pspace = SingleFinitePSpace(name, dist)
+    if any(is_random(arg) for arg in args):
+        from sympy.stats.compound_rv import CompoundPSpace, CompoundDistribution
+        pspace = CompoundPSpace(name, CompoundDistribution(dist))
+    return pspace.value
 
 class FiniteDistributionHandmade(SingleFiniteDistribution):
 
@@ -70,13 +66,27 @@ class FiniteDistributionHandmade(SingleFiniteDistribution):
         for p in density.values():
             _value_check((p >= 0, p <= 1),
                         "Probability at a point must be between 0 and 1.")
-        _value_check(Eq(sum(density.values()), 1), "Total Probability must be 1.")
+        val = sum(density.values())
+        _value_check(Eq(val, 1) != S.false, "Total Probability must be 1.")
 
-def FiniteRV(name, density):
-    """
+def FiniteRV(name, density, **kwargs):
+    r"""
     Create a Finite Random Variable given a dict representing the density.
 
-    Returns a RandomSymbol.
+    Parameters
+    ==========
+
+    name : Symbol
+        Represents name of the random variable.
+    density: A dict
+        Dictionary conatining the pdf of finite distribution
+    check : bool
+        If True, it will check whether the given density
+        integrates to 1 over the given set. If False, it
+        will not perform this check. Default is False.
+
+    Examples
+    ========
 
     >>> from sympy.stats import FiniteRV, P, E
 
@@ -87,18 +97,44 @@ def FiniteRV(name, density):
     2.00000000000000
     >>> P(X >= 2)
     0.700000000000000
+
+    Returns
+    =======
+
+    RandomSymbol
+
     """
-    return rv(name, FiniteDistributionHandmade, density)
+    # have a default of False while `rv` should have a default of True
+    kwargs['check'] = kwargs.pop('check', False)
+    return rv(name, FiniteDistributionHandmade, density, **kwargs)
 
 class DiscreteUniformDistribution(SingleFiniteDistribution):
+
+    @staticmethod
+    def check(*args):
+        # not using _value_check since there is a
+        # suggestion for the user
+        if len(set(args)) != len(args):
+            from sympy.utilities.iterables import multiset
+            from sympy.utilities.misc import filldedent
+            weights = multiset(args)
+            n = Integer(len(args))
+            for k in weights:
+                weights[k] /= n
+            raise ValueError(filldedent("""
+                Repeated args detected but set expected. For a
+                distribution having different weights for each
+                item use the following:""") + (
+                '\nS("FiniteRV(%s, %s)")' % ("'X'", weights)))
+
     @property
     def p(self):
         return Rational(1, len(self.args))
 
-    @property
+    @property  # type: ignore
     @cacheit
     def dict(self):
-        return dict((k, self.p) for k in self.set)
+        return {k: self.p for k in self.set}
 
     @property
     def set(self):
@@ -110,18 +146,17 @@ class DiscreteUniformDistribution(SingleFiniteDistribution):
         else:
             return S.Zero
 
-    def _sample_random(self, size):
-        x = Symbol('x')
-        return ArrayComprehensionMap(lambda: self.args[random.randint(0, len(self.args)-1)], (x, 0, size)).doit()
-
-
 
 def DiscreteUniform(name, items):
-    """
+    r"""
     Create a Finite Random Variable representing a uniform distribution over
     the input set.
 
-    Returns a RandomSymbol.
+    Parameters
+    ==========
+
+    items: list/tuple
+        Items over which Uniform distribution is to be made
 
     Examples
     ========
@@ -137,6 +172,11 @@ def DiscreteUniform(name, items):
     >>> density(Y).dict
     {0: 1/5, 1: 1/5, 2: 1/5, 3: 1/5, 4: 1/5}
 
+    Returns
+    =======
+
+    RandomSymbol
+
     References
     ==========
 
@@ -175,17 +215,21 @@ class DieDistribution(SingleFiniteDistribution):
 
     def pmf(self, x):
         x = sympify(x)
-        if not (x.is_number or x.is_Symbol or isinstance(x, RandomSymbol)):
+        if not (x.is_number or x.is_Symbol or is_random(x)):
             raise ValueError("'x' expected as an argument of type 'number' or 'Symbol' or , "
                         "'RandomSymbol' not %s" % (type(x)))
         cond = Ge(x, 1) & Le(x, self.sides) & Contains(x, S.Integers)
         return Piecewise((S.One/self.sides, cond), (S.Zero, True))
 
 def Die(name, sides=6):
-    """
+    r"""
     Create a Finite Random Variable representing a fair die.
 
-    Returns a RandomSymbol.
+    Parameters
+    ==========
+
+    sides: Integer
+        Represents the number of sides of the Die, by default is 6
 
     Examples
     ========
@@ -207,6 +251,11 @@ def Die(name, sides=6):
     Density(DieDistribution(n))
     >>> density(Dn).dict.subs(n, 4).doit()
     {1: 1/4, 2: 1/4, 3: 1/4, 4: 1/4}
+
+    Returns
+    =======
+
+    RandomSymbol
     """
 
     return rv(name, DieDistribution, sides)
@@ -222,19 +271,31 @@ class BernoulliDistribution(SingleFiniteDistribution):
 
     @property
     def set(self):
-        return set([self.succ, self.fail])
+        return {self.succ, self.fail}
 
     def pmf(self, x):
-        return Piecewise((self.p, x == self.succ),
-                         (1 - self.p, x == self.fail),
+        if isinstance(self.succ, Symbol) and isinstance(self.fail, Symbol):
+            return Piecewise((self.p, x == self.succ),
+                             (1 - self.p, x == self.fail),
+                             (S.Zero, True))
+        return Piecewise((self.p, Eq(x, self.succ)),
+                         (1 - self.p, Eq(x, self.fail)),
                          (S.Zero, True))
 
 
 def Bernoulli(name, p, succ=1, fail=0):
-    """
+    r"""
     Create a Finite Random Variable representing a Bernoulli process.
 
-    Returns a RandomSymbol
+    Parameters
+    ==========
+
+    p : Rational number between 0 and 1
+       Represents probability of success
+    succ : Integer/symbol/string
+       Represents event of success
+    fail : Integer/symbol/string
+       Represents event of failure
 
     Examples
     ========
@@ -250,6 +311,11 @@ def Bernoulli(name, p, succ=1, fail=0):
     >>> density(X).dict
     {Heads: 1/2, Tails: 1/2}
 
+    Returns
+    =======
+
+    RandomSymbol
+
     References
     ==========
 
@@ -262,12 +328,14 @@ def Bernoulli(name, p, succ=1, fail=0):
 
 
 def Coin(name, p=S.Half):
-    """
+    r"""
     Create a Finite Random Variable representing a Coin toss.
 
-    Probability p is the chance of gettings "Heads." Half by default
+    Parameters
+    ==========
 
-    Returns a RandomSymbol.
+    p : Rational Numeber between 0 and 1
+      Represents probability of getting "Heads", by default is Half
 
     Examples
     ========
@@ -283,6 +351,11 @@ def Coin(name, p=S.Half):
     >>> density(C2).dict
     {H: 3/5, T: 2/5}
 
+    Returns
+    =======
+
+    RandomSymbol
+
     See Also
     ========
 
@@ -328,25 +401,35 @@ class BinomialDistribution(SingleFiniteDistribution):
     def pmf(self, x):
         n, p = self.n, self.p
         x = sympify(x)
-        if not (x.is_number or x.is_Symbol or isinstance(x, RandomSymbol)):
+        if not (x.is_number or x.is_Symbol or is_random(x)):
             raise ValueError("'x' expected as an argument of type 'number' or 'Symbol' or , "
                         "'RandomSymbol' not %s" % (type(x)))
         cond = Ge(x, 0) & Le(x, n) & Contains(x, S.Integers)
         return Piecewise((binomial(n, x) * p**x * (1 - p)**(n - x), cond), (S.Zero, True))
 
-    @property
+    @property  # type: ignore
     @cacheit
     def dict(self):
         if self.is_symbolic:
             return Density(self)
-        return dict((k*self.succ + (self.n-k)*self.fail, self.pmf(k))
-                    for k in range(0, self.n + 1))
+        return {k*self.succ + (self.n-k)*self.fail: self.pmf(k)
+                    for k in range(0, self.n + 1)}
 
 def Binomial(name, n, p, succ=1, fail=0):
-    """
+    r"""
     Create a Finite Random Variable representing a binomial distribution.
 
-    Returns a RandomSymbol.
+    Parameters
+    ==========
+
+    n : Positive Integer
+      Represents number of trials
+    p : Rational Number between 0 and 1
+      Represents probability of success
+    succ : Integer/symbol/string
+      Represents event of success, by default is 1
+    fail : Integer/symbol/string
+      Represents event of failure, by default is 0
 
     Examples
     ========
@@ -366,6 +449,11 @@ def Binomial(name, n, p, succ=1, fail=0):
     >>> density(X).dict.subs(n, 4).doit()
     {0: 1/16, 1: 1/4, 2: 3/8, 3: 1/4, 4: 1/16}
 
+    Returns
+    =======
+
+    RandomSymbol
+
     References
     ==========
 
@@ -413,28 +501,33 @@ class BetaBinomialDistribution(SingleFiniteDistribution):
         n, a, b = self.n, self.alpha, self.beta
         return binomial(n, k) * beta_fn(k + a, n - k + b) / beta_fn(a, b)
 
-    def _sample_pymc3(self, size):
-        n, a, b = int(self.n), float(self.alpha), float(self.beta)
-        with pymc3.Model():
-            pymc3.BetaBinomial('X', alpha=a, beta=b, n=n)
-            return pymc3.sample(size, chains=1, progressbar=False)[:]['X']
 
 def BetaBinomial(name, n, alpha, beta):
-    """
+    r"""
     Create a Finite Random Variable representing a Beta-binomial distribution.
 
-    Returns a RandomSymbol.
+    Parameters
+    ==========
+
+    n : Positive Integer
+      Represents number of trials
+    alpha : Real positive number
+    beta : Real positive number
 
     Examples
     ========
 
     >>> from sympy.stats import BetaBinomial, density
-    >>> from sympy import S
 
     >>> X = BetaBinomial('X', 2, 1, 1)
     >>> density(X).dict
     {0: 1/3, 1: 2*beta(2, 2), 2: 1/3}
 
+    Returns
+    =======
+
+    RandomSymbol
+
     References
     ==========
 
@@ -475,32 +568,42 @@ class HypergeometricDistribution(SingleFiniteDistribution):
         N, m, n = self.N, self.m, self.n
         if self.is_symbolic:
             return Intersection(S.Naturals0, Interval(self.low, self.high))
-        return set([i for i in range(max(0, n + m - N), min(n, m) + 1)])
+        return {i for i in range(max(0, n + m - N), min(n, m) + 1)}
 
     def pmf(self, k):
         N, m, n = self.N, self.m, self.n
         return S(binomial(m, k) * binomial(N - m, n - k))/binomial(N, n)
 
-    def _sample_scipy(self, size):
-        N, m, n = int(self.N), int(self.m), int(self.n)
-        return scipy.stats.hypergeom.rvs(M=m, n=n, N=N, size=size)
 
 def Hypergeometric(name, N, m, n):
-    """
+    r"""
     Create a Finite Random Variable representing a hypergeometric distribution.
 
-    Returns a RandomSymbol.
+    Parameters
+    ==========
+
+    N : Positive Integer
+      Represents finite population of size N.
+    m : Positive Integer
+      Represents number of trials with required feature.
+    n : Positive Integer
+      Represents numbers of draws.
+
 
     Examples
     ========
 
     >>> from sympy.stats import Hypergeometric, density
-    >>> from sympy import S
 
     >>> X = Hypergeometric('X', 10, 5, 3) # 10 marbles, 5 white (success), 3 draws
     >>> density(X).dict
     {0: 1/12, 1: 5/12, 2: 5/12, 3: 1/12}
 
+    Returns
+    =======
+
+    RandomSymbol
+
     References
     ==========
 
@@ -515,7 +618,7 @@ class RademacherDistribution(SingleFiniteDistribution):
 
     @property
     def set(self):
-        return set([-1, 1])
+        return {-1, 1}
 
     @property
     def pmf(self):
@@ -523,11 +626,9 @@ class RademacherDistribution(SingleFiniteDistribution):
         return Lambda(k, Piecewise((S.Half, Or(Eq(k, -1), Eq(k, 1))), (S.Zero, True)))
 
 def Rademacher(name):
-    """
+    r"""
     Create a Finite Random Variable representing a Rademacher distribution.
 
-    Return a RandomSymbol.
-
     Examples
     ========
 
@@ -537,6 +638,11 @@ def Rademacher(name):
     >>> density(X).dict
     {-1: 1/2, 1: 1/2}
 
+    Returns
+    =======
+
+    RandomSymbol
+
     See Also
     ========
 
