@@ -1,11 +1,9 @@
-from __future__ import print_function, division
-
 from functools import wraps
 
 from sympy import S, pi, I, Rational, Wild, cacheit, sympify
 from sympy.core.function import Function, ArgumentIndexError
+from sympy.core.logic import fuzzy_or, fuzzy_not
 from sympy.core.power import Pow
-from sympy.core.compatibility import range
 from sympy.functions.combinatorial.factorials import factorial
 from sympy.functions.elementary.trigonometric import sin, cos, csc, cot
 from sympy.functions.elementary.complexes import Abs
@@ -30,27 +28,28 @@ from sympy.polys.orthopolys import spherical_bessel_fn as fn
 
 class BesselBase(Function):
     """
-    Abstract base class for bessel-type functions.
+    Abstract base class for Bessel-type functions.
 
     This class is meant to reduce code duplication.
-    All Bessel type functions can 1) be differentiated, and the derivatives
-    expressed in terms of similar functions and 2) be rewritten in terms
-    of other bessel-type functions.
+    All Bessel-type functions can 1) be differentiated, with the derivatives
+    expressed in terms of similar functions, and 2) be rewritten in terms
+    of other Bessel-type functions.
 
-    Here "bessel-type functions" are assumed to have one complex parameter.
+    Here, Bessel-type functions are assumed to have one complex parameter.
 
     To use this base class, define class attributes ``_a`` and ``_b`` such that
     ``2*F_n' = -_a*F_{n+1} + b*F_{n-1}``.
+
     """
 
     @property
     def order(self):
-        """ The order of the bessel-type function. """
+        """ The order of the Bessel-type function. """
         return self.args[0]
 
     @property
     def argument(self):
-        """ The argument of the bessel-type function. """
+        """ The argument of the Bessel-type function. """
         return self.args[1]
 
     @classmethod
@@ -65,21 +64,34 @@ class BesselBase(Function):
 
     def _eval_conjugate(self):
         z = self.argument
-        if (z.is_real and z.is_negative) is False:
+        if z.is_extended_negative is False:
             return self.__class__(self.order.conjugate(), z.conjugate())
 
+    def _eval_is_meromorphic(self, x, a):
+        nu, z = self.order, self.argument
+
+        if nu.has(x):
+            return False
+        if not z._eval_is_meromorphic(x, a):
+            return None
+        z0 = z.subs(x, a)
+        if nu.is_integer:
+            if isinstance(self, (besselj, besseli, hn1, hn2, jn, yn)) or not nu.is_zero:
+                return fuzzy_not(z0.is_infinite)
+        return fuzzy_not(fuzzy_or([z0.is_zero, z0.is_infinite]))
+
     def _eval_expand_func(self, **hints):
         nu, z, f = self.order, self.argument, self.__class__
-        if nu.is_real:
-            if (nu - 1).is_positive:
+        if nu.is_extended_real:
+            if (nu - 1).is_extended_positive:
                 return (-self._a*self._b*f(nu - 2, z)._eval_expand_func() +
                         2*self._a*(nu - 1)*f(nu - 1, z)._eval_expand_func()/z)
-            elif (nu + 1).is_negative:
+            elif (nu + 1).is_extended_negative:
                 return (2*self._b*(nu + 1)*f(nu + 1, z)._eval_expand_func()/z -
                         self._a*self._b*f(nu + 2, z)._eval_expand_func())
         return self
 
-    def _eval_simplify(self, ratio, measure, rational, inverse):
+    def _eval_simplify(self, **kwargs):
         from sympy.simplify.simplify import besselsimp
         return besselsimp(self)
 
@@ -88,7 +100,10 @@ class besselj(BesselBase):
     r"""
     Bessel function of the first kind.
 
-    The Bessel `J` function of order `\nu` is defined to be the function
+    Explanation
+    ===========
+
+    The Bessel $J$ function of order $\nu$ is defined to be the function
     satisfying Bessel's differential equation
 
     .. math ::
@@ -100,7 +115,7 @@ class besselj(BesselBase):
     .. math ::
         J_\nu(z) = z^\nu \left(\frac{1}{\Gamma(\nu + 1) 2^\nu} + O(z^2) \right),
 
-    if :math:`\nu` is not a negative integer. If :math:`\nu=-n \in \mathbb{Z}_{<0}`
+    if $\nu$ is not a negative integer. If $\nu=-n \in \mathbb{Z}_{<0}$
     *is* a negative integer, then the definition is
 
     .. math ::
@@ -147,6 +162,7 @@ class besselj(BesselBase):
            Approximations, Volume 1
     .. [3] https://en.wikipedia.org/wiki/Bessel_function
     .. [4] http://functions.wolfram.com/Bessel-TypeFunctions/BesselJ/
+
     """
 
     _a = S.One
@@ -170,7 +186,7 @@ class besselj(BesselBase):
             return (z)**nu*(-z)**(-nu)*besselj(nu, -z)
         if nu.is_integer:
             if nu.could_extract_minus_sign():
-                return S(-1)**(-nu)*besselj(-nu, z)
+                return S.NegativeOne**(-nu)*besselj(-nu, z)
             newz = z.extract_multiplicatively(I)
             if newz:  # NOTE we don't want to change the function if z==0
                 return I**(nu)*besseli(nu, newz)
@@ -200,9 +216,9 @@ class besselj(BesselBase):
     def _eval_rewrite_as_jn(self, nu, z, **kwargs):
         return sqrt(2*z/pi)*jn(nu - S.Half, self.argument)
 
-    def _eval_is_real(self):
+    def _eval_is_extended_real(self):
         nu, z = self.args
-        if nu.is_integer and z.is_real:
+        if nu.is_integer and z.is_extended_real:
             return True
 
     def _sage_(self):
@@ -214,16 +230,19 @@ class bessely(BesselBase):
     r"""
     Bessel function of the second kind.
 
-    The Bessel `Y` function of order `\nu` is defined as
+    Explanation
+    ===========
+
+    The Bessel $Y$ function of order $\nu$ is defined as
 
     .. math ::
         Y_\nu(z) = \lim_{\mu \to \nu} \frac{J_\mu(z) \cos(\pi \mu)
                                             - J_{-\mu}(z)}{\sin(\pi \mu)},
 
-    where :math:`J_\mu(z)` is the Bessel function of the first kind.
+    where $J_\mu(z)$ is the Bessel function of the first kind.
 
     It is a solution to Bessel's equation, and linearly independent from
-    :math:`J_\nu`.
+    $J_\nu$.
 
     Examples
     ========
@@ -265,7 +284,7 @@ class bessely(BesselBase):
 
         if nu.is_integer:
             if nu.could_extract_minus_sign():
-                return S(-1)**(-nu)*bessely(-nu, z)
+                return S.NegativeOne**(-nu)*bessely(-nu, z)
 
     def _eval_rewrite_as_besselj(self, nu, z, **kwargs):
         if nu.is_integer is False:
@@ -279,7 +298,7 @@ class bessely(BesselBase):
     def _eval_rewrite_as_yn(self, nu, z, **kwargs):
         return sqrt(2*z/pi) * yn(nu - S.Half, self.argument)
 
-    def _eval_is_real(self):
+    def _eval_is_extended_real(self):
         nu, z = self.args
         if nu.is_integer and z.is_positive:
             return True
@@ -293,7 +312,10 @@ class besseli(BesselBase):
     r"""
     Modified Bessel function of the first kind.
 
-    The Bessel I function is a solution to the modified Bessel equation
+    Explanation
+    ===========
+
+    The Bessel $I$ function is a solution to the modified Bessel equation
 
     .. math ::
         z^2 \frac{\mathrm{d}^2 w}{\mathrm{d}z^2}
@@ -304,7 +326,7 @@ class besseli(BesselBase):
     .. math ::
         I_\nu(z) = i^{-\nu} J_\nu(iz),
 
-    where :math:`J_\nu(z)` is the Bessel function of the first kind.
+    where $J_\nu(z)$ is the Bessel function of the first kind.
 
     Examples
     ========
@@ -340,9 +362,8 @@ class besseli(BesselBase):
                 return S.ComplexInfinity
             elif nu.is_imaginary:
                 return S.NaN
-        if z.is_imaginary:
-            if im(z) is S.Infinity or im(z) is S.NegativeInfinity:
-                return S.Zero
+        if im(z) is S.Infinity or im(z) is S.NegativeInfinity:
+            return S.Zero
 
         if z.could_extract_minus_sign():
             return (z)**nu*(-z)**(-nu)*besseli(nu, -z)
@@ -379,9 +400,9 @@ class besseli(BesselBase):
     def _eval_rewrite_as_jn(self, nu, z, **kwargs):
         return self._eval_rewrite_as_besselj(*self.args).rewrite(jn)
 
-    def _eval_is_real(self):
+    def _eval_is_extended_real(self):
         nu, z = self.args
-        if nu.is_integer and z.is_real:
+        if nu.is_integer and z.is_extended_real:
             return True
 
     def _sage_(self):
@@ -393,16 +414,19 @@ class besselk(BesselBase):
     r"""
     Modified Bessel function of the second kind.
 
-    The Bessel K function of order :math:`\nu` is defined as
+    Explanation
+    ===========
+
+    The Bessel $K$ function of order $\nu$ is defined as
 
     .. math ::
         K_\nu(z) = \lim_{\mu \to \nu} \frac{\pi}{2}
                    \frac{I_{-\mu}(z) -I_\mu(z)}{\sin(\pi \mu)},
 
-    where :math:`I_\mu(z)` is the modified Bessel function of the first kind.
+    where $I_\mu(z)$ is the modified Bessel function of the first kind.
 
     It is a solution of the modified Bessel equation, and linearly independent
-    from :math:`Y_\nu`.
+    from $Y_\nu$.
 
     Examples
     ========
@@ -436,9 +460,8 @@ class besselk(BesselBase):
                 return S.ComplexInfinity
             elif re(nu).is_zero:
                 return S.NaN
-        if z.is_imaginary:
-            if im(z) is S.Infinity or im(z) is S.NegativeInfinity:
-                return S.Zero
+        if z in (S.Infinity, I*S.Infinity, I*S.NegativeInfinity):
+            return S.Zero
 
         if nu.is_integer:
             if nu.could_extract_minus_sign():
@@ -463,7 +486,7 @@ class besselk(BesselBase):
         if ay:
             return ay.rewrite(yn)
 
-    def _eval_is_real(self):
+    def _eval_is_extended_real(self):
         nu, z = self.args
         if nu.is_integer and z.is_positive:
             return True
@@ -477,13 +500,16 @@ class hankel1(BesselBase):
     r"""
     Hankel function of the first kind.
 
+    Explanation
+    ===========
+
     This function is defined as
 
     .. math ::
         H_\nu^{(1)} = J_\nu(z) + iY_\nu(z),
 
-    where :math:`J_\nu(z)` is the Bessel function of the first kind, and
-    :math:`Y_\nu(z)` is the Bessel function of the second kind.
+    where $J_\nu(z)$ is the Bessel function of the first kind, and
+    $Y_\nu(z)$ is the Bessel function of the second kind.
 
     It is a solution to Bessel's equation.
 
@@ -512,7 +538,7 @@ class hankel1(BesselBase):
 
     def _eval_conjugate(self):
         z = self.argument
-        if (z.is_real and z.is_negative) is False:
+        if z.is_extended_negative is False:
             return hankel2(self.order.conjugate(), z.conjugate())
 
 
@@ -520,16 +546,19 @@ class hankel2(BesselBase):
     r"""
     Hankel function of the second kind.
 
+    Explanation
+    ===========
+
     This function is defined as
 
     .. math ::
         H_\nu^{(2)} = J_\nu(z) - iY_\nu(z),
 
-    where :math:`J_\nu(z)` is the Bessel function of the first kind, and
-    :math:`Y_\nu(z)` is the Bessel function of the second kind.
+    where $J_\nu(z)$ is the Bessel function of the first kind, and
+    $Y_\nu(z)$ is the Bessel function of the second kind.
 
     It is a solution to Bessel's equation, and linearly independent from
-    :math:`H_\nu^{(1)}`.
+    $H_\nu^{(1)}$.
 
     Examples
     ========
@@ -556,7 +585,7 @@ class hankel2(BesselBase):
 
     def _eval_conjugate(self):
         z = self.argument
-        if (z.is_real and z.is_negative) is False:
+        if z.is_extended_negative is False:
             return hankel1(self.order.conjugate(), z.conjugate())
 
 
@@ -576,7 +605,8 @@ class SphericalBesselBase(BesselBase):
     since spherical Bessel functions differ from the ordinary
     ones just by a slight change in order.
 
-    To use this class, define the ``_rewrite`` and ``_expand`` methods.
+    To use this class, define the ``_rewrite()`` and ``_expand()`` methods.
+
     """
 
     def _expand(self, **hints):
@@ -616,6 +646,9 @@ class jn(SphericalBesselBase):
     r"""
     Spherical Bessel function of the first kind.
 
+    Explanation
+    ===========
+
     This function is a solution to the spherical Bessel equation
 
     .. math ::
@@ -627,21 +660,20 @@ class jn(SphericalBesselBase):
     .. math ::
         j_\nu(z) = \sqrt{\frac{\pi}{2z}} J_{\nu + \frac{1}{2}}(z),
 
-    where :math:`J_\nu(z)` is the Bessel function of the first kind.
+    where $J_\nu(z)$ is the Bessel function of the first kind.
 
     The spherical Bessel functions of integral order are
     calculated using the formula:
 
     .. math:: j_n(z) = f_n(z) \sin{z} + (-1)^{n+1} f_{-n-1}(z) \cos{z},
 
-    where the coefficients :math:`f_n(z)` are available as
-    :func:`polys.orthopolys.spherical_bessel_fn`.
+    where the coefficients $f_n(z)$ are available as
+    :func:`sympy.polys.orthopolys.spherical_bessel_fn`.
 
     Examples
     ========
 
     >>> from sympy import Symbol, jn, sin, cos, expand_func, besselj, bessely
-    >>> from sympy import simplify
     >>> z = Symbol("z")
     >>> nu = Symbol("nu", integer=True)
     >>> print(expand_func(jn(0, z)))
@@ -668,6 +700,18 @@ class jn(SphericalBesselBase):
     .. [1] http://dlmf.nist.gov/10.47
 
     """
+    @classmethod
+    def eval(cls, nu, z):
+        if z.is_zero:
+            if nu.is_zero:
+                return S.One
+            elif nu.is_integer:
+                if nu.is_positive:
+                    return S.Zero
+                else:
+                    return S.ComplexInfinity
+        if z in (S.NegativeInfinity, S.Infinity):
+            return S.Zero
 
     def _rewrite(self):
         return self._eval_rewrite_as_besselj(self.order, self.argument)
@@ -689,15 +733,18 @@ class yn(SphericalBesselBase):
     r"""
     Spherical Bessel function of the second kind.
 
+    Explanation
+    ===========
+
     This function is another solution to the spherical Bessel equation, and
-    linearly independent from :math:`j_n`. It can be defined as
+    linearly independent from $j_n$. It can be defined as
 
     .. math ::
         y_\nu(z) = \sqrt{\frac{\pi}{2z}} Y_{\nu + \frac{1}{2}}(z),
 
-    where :math:`Y_\nu(z)` is the Bessel function of the second kind.
+    where $Y_\nu(z)$ is the Bessel function of the second kind.
 
-    For integral orders :math:`n`, :math:`y_n` is calculated using the formula:
+    For integral orders $n$, $y_n$ is calculated using the formula:
 
     .. math:: y_n(z) = (-1)^{n+1} j_{-n-1}(z)
 
@@ -808,14 +855,17 @@ class hn1(SphericalHankelBase):
     r"""
     Spherical Hankel function of the first kind.
 
+    Explanation
+    ===========
+
     This function is defined as
 
     .. math:: h_\nu^(1)(z) = j_\nu(z) + i y_\nu(z),
 
-    where :math:`j_\nu(z)` and :math:`y_\nu(z)` are the spherical
+    where $j_\nu(z)$ and $y_\nu(z)$ are the spherical
     Bessel function of the first and second kinds.
 
-    For integral orders :math:`n`, :math:`h_n^(1)` is calculated using the formula:
+    For integral orders $n$, $h_n^(1)$ is calculated using the formula:
 
     .. math:: h_n^(1)(z) = j_{n}(z) + i (-1)^{n+1} j_{-n-1}(z)
 
@@ -861,14 +911,17 @@ class hn2(SphericalHankelBase):
     r"""
     Spherical Hankel function of the second kind.
 
+    Explanation
+    ===========
+
     This function is defined as
 
     .. math:: h_\nu^(2)(z) = j_\nu(z) - i y_\nu(z),
 
-    where :math:`j_\nu(z)` and :math:`y_\nu(z)` are the spherical
+    where $j_\nu(z)$ and $y_\nu(z)$ are the spherical
     Bessel function of the first and second kinds.
 
-    For integral orders :math:`n`, :math:`h_n^(2)` is calculated using the formula:
+    For integral orders $n$, $h_n^(2)$ is calculated using the formula:
 
     .. math:: h_n^(2)(z) = j_{n} - i (-1)^{n+1} j_{-n-1}(z)
 
@@ -914,9 +967,13 @@ def jn_zeros(n, k, method="sympy", dps=15):
     """
     Zeros of the spherical Bessel function of the first kind.
 
-    This returns an array of zeros of jn up to the k-th zero.
+    Explanation
+    ===========
 
-    * method = "sympy": uses :func:`mpmath.besseljzero`
+    This returns an array of zeros of $jn$ up to the $k$-th zero.
+
+    * method = "sympy": uses `mpmath.besseljzero
+      <http://mpmath.org/doc/current/functions/bessel.html#mpmath.besseljzero>`_
     * method = "scipy": uses the
       `SciPy's sph_jn <http://docs.scipy.org/doc/scipy/reference/generated/scipy.special.jn_zeros.html>`_
       and
@@ -924,9 +981,9 @@ def jn_zeros(n, k, method="sympy", dps=15):
       to find all
       roots, which is faster than computing the zeros using a general
       numerical solver, but it requires SciPy and only works with low
-      precision floating point numbers.  [The function used with
-      method="sympy" is a recent addition to mpmath, before that a general
-      solver was used.]
+      precision floating point numbers. (The function used with
+      method="sympy" is a recent addition to mpmath; before that a general
+      solver was used.)
 
     Examples
     ========
@@ -939,6 +996,17 @@ def jn_zeros(n, k, method="sympy", dps=15):
     ========
 
     jn, yn, besselj, besselk, bessely
+
+    Parameters
+    ==========
+
+    n : integer
+        order of Bessel function
+
+    k : integer
+        number of zeros to return
+
+
     """
     from math import pi
 
@@ -985,33 +1053,22 @@ class AiryBase(Function):
     Abstract base class for Airy functions.
 
     This class is meant to reduce code duplication.
+
     """
 
     def _eval_conjugate(self):
         return self.func(self.args[0].conjugate())
 
-    def _eval_is_real(self):
-        return self.args[0].is_real
-
-    def _as_real_imag(self, deep=True, **hints):
-        if self.args[0].is_real:
-            if deep:
-                hints['complex'] = False
-                return (self.expand(deep, **hints), S.Zero)
-            else:
-                return (self, S.Zero)
-        if deep:
-            re, im = self.args[0].expand(deep, **hints).as_real_imag()
-        else:
-            re, im = self.args[0].as_real_imag()
-        return (re, im)
+    def _eval_is_extended_real(self):
+        return self.args[0].is_extended_real
 
     def as_real_imag(self, deep=True, **hints):
-        x, y = self._as_real_imag(deep=deep, **hints)
-        sq = -y**2/x**2
-        re = S.Half*(self.func(x+x*sqrt(sq))+self.func(x-x*sqrt(sq)))
-        im = x/(2*y) * sqrt(sq) * (self.func(x-x*sqrt(sq)) - self.func(x+x*sqrt(sq)))
-        return (re, im)
+        z = self.args[0]
+        zc = z.conjugate()
+        f = self.func
+        u = (f(z)+f(zc))/2
+        v = I*(f(zc)-f(z))/2
+        return u, v
 
     def _eval_expand_complex(self, deep=True, **hints):
         re_part, im_part = self.as_real_imag(deep=deep, **hints)
@@ -1020,15 +1077,18 @@ class AiryBase(Function):
 
 class airyai(AiryBase):
     r"""
-    The Airy function `\operatorname{Ai}` of the first kind.
+    The Airy function $\operatorname{Ai}$ of the first kind.
 
-    The Airy function `\operatorname{Ai}(z)` is defined to be the function
+    Explanation
+    ===========
+
+    The Airy function $\operatorname{Ai}(z)$ is defined to be the function
     satisfying Airy's differential equation
 
     .. math::
         \frac{\mathrm{d}^2 w(z)}{\mathrm{d}z^2} - z w(z) = 0.
 
-    Equivalently, for real `z`
+    Equivalently, for real $z$
 
     .. math::
         \operatorname{Ai}(z) := \frac{1}{\pi}
@@ -1061,7 +1121,7 @@ class airyai(AiryBase):
     >>> conjugate(airyai(z))
     airyai(conjugate(z))
 
-    Differentiation with respect to z is supported:
+    Differentiation with respect to $z$ is supported:
 
     >>> from sympy import diff
     >>> diff(airyai(z), z)
@@ -1081,7 +1141,7 @@ class airyai(AiryBase):
     >>> airyai(-2).evalf(50)
     0.22740742820168557599192443603787379946077222541710
 
-    Rewrite Ai(z) in terms of hypergeometric functions:
+    Rewrite $\operatorname{Ai}(z)$ in terms of hypergeometric functions:
 
     >>> from sympy import hyper
     >>> airyai(z).rewrite(hyper)
@@ -1101,6 +1161,7 @@ class airyai(AiryBase):
     .. [2] http://dlmf.nist.gov/9
     .. [3] http://www.encyclopediaofmath.org/index.php/Airy_functions
     .. [4] http://mathworld.wolfram.com/AiryFunctions.html
+
     """
 
     nargs = 1
@@ -1115,8 +1176,10 @@ class airyai(AiryBase):
                 return S.Zero
             elif arg is S.NegativeInfinity:
                 return S.Zero
-            elif arg is S.Zero:
+            elif arg.is_zero:
                 return S.One / (3**Rational(2, 3) * gamma(Rational(2, 3)))
+        if arg.is_zero:
+            return S.One / (3**Rational(2, 3) * gamma(Rational(2, 3)))
 
     def fdiff(self, argindex=1):
         if argindex == 1:
@@ -1133,10 +1196,10 @@ class airyai(AiryBase):
             x = sympify(x)
             if len(previous_terms) > 1:
                 p = previous_terms[-1]
-                return ((3**(S(1)/3)*x)**(-n)*(3**(S(1)/3)*x)**(n + 1)*sin(pi*(2*n/3 + S(4)/3))*factorial(n) *
-                        gamma(n/3 + S(2)/3)/(sin(pi*(2*n/3 + S(2)/3))*factorial(n + 1)*gamma(n/3 + S(1)/3)) * p)
+                return ((3**Rational(1, 3)*x)**(-n)*(3**Rational(1, 3)*x)**(n + 1)*sin(pi*(n*Rational(2, 3) + Rational(4, 3)))*factorial(n) *
+                        gamma(n/3 + Rational(2, 3))/(sin(pi*(n*Rational(2, 3) + Rational(2, 3)))*factorial(n + 1)*gamma(n/3 + Rational(1, 3))) * p)
             else:
-                return (S.One/(3**(S(2)/3)*pi) * gamma((n+S.One)/S(3)) * sin(2*pi*(n+S.One)/S(3)) /
+                return (S.One/(3**Rational(2, 3)*pi) * gamma((n+S.One)/S(3)) * sin(2*pi*(n+S.One)/S(3)) /
                         factorial(n) * (root(3, 3)*x)**n)
 
     def _eval_rewrite_as_besselj(self, z, **kwargs):
@@ -1156,9 +1219,9 @@ class airyai(AiryBase):
             return ot*(Pow(a, ot)*besseli(-ot, tt*a) - z*Pow(a, -ot)*besseli(ot, tt*a))
 
     def _eval_rewrite_as_hyper(self, z, **kwargs):
-        pf1 = S.One / (3**(S(2)/3)*gamma(S(2)/3))
-        pf2 = z / (root(3, 3)*gamma(S(1)/3))
-        return pf1 * hyper([], [S(2)/3], z**3/9) - pf2 * hyper([], [S(4)/3], z**3/9)
+        pf1 = S.One / (3**Rational(2, 3)*gamma(Rational(2, 3)))
+        pf2 = z / (root(3, 3)*gamma(Rational(1, 3)))
+        return pf1 * hyper([], [Rational(2, 3)], z**3/9) - pf2 * hyper([], [Rational(4, 3)], z**3/9)
 
     def _eval_expand_func(self, **hints):
         arg = self.args[0]
@@ -1186,15 +1249,18 @@ class airyai(AiryBase):
 
 class airybi(AiryBase):
     r"""
-    The Airy function `\operatorname{Bi}` of the second kind.
+    The Airy function $\operatorname{Bi}$ of the second kind.
 
-    The Airy function `\operatorname{Bi}(z)` is defined to be the function
+    Explanation
+    ===========
+
+    The Airy function $\operatorname{Bi}(z)$ is defined to be the function
     satisfying Airy's differential equation
 
     .. math::
         \frac{\mathrm{d}^2 w(z)}{\mathrm{d}z^2} - z w(z) = 0.
 
-    Equivalently, for real `z`
+    Equivalently, for real $z$
 
     .. math::
         \operatorname{Bi}(z) := \frac{1}{\pi}
@@ -1229,7 +1295,7 @@ class airybi(AiryBase):
     >>> conjugate(airybi(z))
     airybi(conjugate(z))
 
-    Differentiation with respect to z is supported:
+    Differentiation with respect to $z$ is supported:
 
     >>> from sympy import diff
     >>> diff(airybi(z), z)
@@ -1249,7 +1315,7 @@ class airybi(AiryBase):
     >>> airybi(-2).evalf(50)
     -0.41230258795639848808323405461146104203453483447240
 
-    Rewrite Bi(z) in terms of hypergeometric functions:
+    Rewrite $\operatorname{Bi}(z)$ in terms of hypergeometric functions:
 
     >>> from sympy import hyper
     >>> airybi(z).rewrite(hyper)
@@ -1269,6 +1335,7 @@ class airybi(AiryBase):
     .. [2] http://dlmf.nist.gov/9
     .. [3] http://www.encyclopediaofmath.org/index.php/Airy_functions
     .. [4] http://mathworld.wolfram.com/AiryFunctions.html
+
     """
 
     nargs = 1
@@ -1283,9 +1350,12 @@ class airybi(AiryBase):
                 return S.Infinity
             elif arg is S.NegativeInfinity:
                 return S.Zero
-            elif arg is S.Zero:
+            elif arg.is_zero:
                 return S.One / (3**Rational(1, 6) * gamma(Rational(2, 3)))
 
+        if arg.is_zero:
+            return S.One / (3**Rational(1, 6) * gamma(Rational(2, 3)))
+
     def fdiff(self, argindex=1):
         if argindex == 1:
             return airybiprime(self.args[0])
@@ -1301,7 +1371,7 @@ class airybi(AiryBase):
             x = sympify(x)
             if len(previous_terms) > 1:
                 p = previous_terms[-1]
-                return (3**(S(1)/3)*x * Abs(sin(2*pi*(n + S.One)/S(3))) * factorial((n - S.One)/S(3)) /
+                return (3**Rational(1, 3)*x * Abs(sin(2*pi*(n + S.One)/S(3))) * factorial((n - S.One)/S(3)) /
                         ((n + S.One) * Abs(cos(2*pi*(n + S.Half)/S(3))) * factorial((n - 2)/S(3))) * p)
             else:
                 return (S.One/(root(3, 6)*pi) * gamma((n + S.One)/S(3)) * Abs(sin(2*pi*(n + S.One)/S(3))) /
@@ -1326,9 +1396,9 @@ class airybi(AiryBase):
             return sqrt(ot)*(b*besseli(-ot, tt*a) + z*c*besseli(ot, tt*a))
 
     def _eval_rewrite_as_hyper(self, z, **kwargs):
-        pf1 = S.One / (root(3, 6)*gamma(S(2)/3))
-        pf2 = z*root(3, 6) / gamma(S(1)/3)
-        return pf1 * hyper([], [S(2)/3], z**3/9) + pf2 * hyper([], [S(4)/3], z**3/9)
+        pf1 = S.One / (root(3, 6)*gamma(Rational(2, 3)))
+        pf2 = z*root(3, 6) / gamma(Rational(1, 3))
+        return pf1 * hyper([], [Rational(2, 3)], z**3/9) + pf2 * hyper([], [Rational(4, 3)], z**3/9)
 
     def _eval_expand_func(self, **hints):
         arg = self.args[0]
@@ -1356,9 +1426,14 @@ class airybi(AiryBase):
 
 class airyaiprime(AiryBase):
     r"""
-    The derivative `\operatorname{Ai}^\prime` of the Airy function of the first kind.
+    The derivative $\operatorname{Ai}^\prime$ of the Airy function of the first
+    kind.
 
-    The Airy function `\operatorname{Ai}^\prime(z)` is defined to be the function
+    Explanation
+    ===========
+
+    The Airy function $\operatorname{Ai}^\prime(z)$ is defined to be the
+    function
 
     .. math::
         \operatorname{Ai}^\prime(z) := \frac{\mathrm{d} \operatorname{Ai}(z)}{\mathrm{d} z}.
@@ -1388,7 +1463,7 @@ class airyaiprime(AiryBase):
     >>> conjugate(airyaiprime(z))
     airyaiprime(conjugate(z))
 
-    Differentiation with respect to z is supported:
+    Differentiation with respect to $z$ is supported:
 
     >>> from sympy import diff
     >>> diff(airyaiprime(z), z)
@@ -1408,7 +1483,7 @@ class airyaiprime(AiryBase):
     >>> airyaiprime(-2).evalf(50)
     0.61825902074169104140626429133247528291577794512415
 
-    Rewrite Ai'(z) in terms of hypergeometric functions:
+    Rewrite $\operatorname{Ai}^\prime(z)$ in terms of hypergeometric functions:
 
     >>> from sympy import hyper
     >>> airyaiprime(z).rewrite(hyper)
@@ -1428,6 +1503,7 @@ class airyaiprime(AiryBase):
     .. [2] http://dlmf.nist.gov/9
     .. [3] http://www.encyclopediaofmath.org/index.php/Airy_functions
     .. [4] http://mathworld.wolfram.com/AiryFunctions.html
+
     """
 
     nargs = 1
@@ -1440,8 +1516,9 @@ class airyaiprime(AiryBase):
                 return S.NaN
             elif arg is S.Infinity:
                 return S.Zero
-            elif arg is S.Zero:
-                return -S.One / (3**Rational(1, 3) * gamma(Rational(1, 3)))
+
+        if arg.is_zero:
+            return S.NegativeOne / (3**Rational(1, 3) * gamma(Rational(1, 3)))
 
     def fdiff(self, argindex=1):
         if argindex == 1:
@@ -1476,9 +1553,9 @@ class airyaiprime(AiryBase):
             return ot * (z**2*c*besseli(tt, tt*a) - b*besseli(-ot, tt*a))
 
     def _eval_rewrite_as_hyper(self, z, **kwargs):
-        pf1 = z**2 / (2*3**(S(2)/3)*gamma(S(2)/3))
-        pf2 = 1 / (root(3, 3)*gamma(S(1)/3))
-        return pf1 * hyper([], [S(5)/3], z**3/9) - pf2 * hyper([], [S(1)/3], z**3/9)
+        pf1 = z**2 / (2*3**Rational(2, 3)*gamma(Rational(2, 3)))
+        pf2 = 1 / (root(3, 3)*gamma(Rational(1, 3)))
+        return pf1 * hyper([], [Rational(5, 3)], z**3/9) - pf2 * hyper([], [Rational(1, 3)], z**3/9)
 
     def _eval_expand_func(self, **hints):
         arg = self.args[0]
@@ -1508,9 +1585,14 @@ class airyaiprime(AiryBase):
 
 class airybiprime(AiryBase):
     r"""
-    The derivative `\operatorname{Bi}^\prime` of the Airy function of the first kind.
+    The derivative $\operatorname{Bi}^\prime$ of the Airy function of the first
+    kind.
 
-    The Airy function `\operatorname{Bi}^\prime(z)` is defined to be the function
+    Explanation
+    ===========
+
+    The Airy function $\operatorname{Bi}^\prime(z)$ is defined to be the
+    function
 
     .. math::
         \operatorname{Bi}^\prime(z) := \frac{\mathrm{d} \operatorname{Bi}(z)}{\mathrm{d} z}.
@@ -1542,7 +1624,7 @@ class airybiprime(AiryBase):
     >>> conjugate(airybiprime(z))
     airybiprime(conjugate(z))
 
-    Differentiation with respect to z is supported:
+    Differentiation with respect to $z$ is supported:
 
     >>> from sympy import diff
     >>> diff(airybiprime(z), z)
@@ -1562,7 +1644,7 @@ class airybiprime(AiryBase):
     >>> airybiprime(-2).evalf(50)
     0.27879516692116952268509756941098324140300059345163
 
-    Rewrite Bi'(z) in terms of hypergeometric functions:
+    Rewrite $\operatorname{Bi}^\prime(z)$ in terms of hypergeometric functions:
 
     >>> from sympy import hyper
     >>> airybiprime(z).rewrite(hyper)
@@ -1582,6 +1664,7 @@ class airybiprime(AiryBase):
     .. [2] http://dlmf.nist.gov/9
     .. [3] http://www.encyclopediaofmath.org/index.php/Airy_functions
     .. [4] http://mathworld.wolfram.com/AiryFunctions.html
+
     """
 
     nargs = 1
@@ -1596,9 +1679,13 @@ class airybiprime(AiryBase):
                 return S.Infinity
             elif arg is S.NegativeInfinity:
                 return S.Zero
-            elif arg is S.Zero:
+            elif arg.is_zero:
                 return 3**Rational(1, 6) / gamma(Rational(1, 3))
 
+        if arg.is_zero:
+            return 3**Rational(1, 6) / gamma(Rational(1, 3))
+
+
     def fdiff(self, argindex=1):
         if argindex == 1:
             return self.args[0]*airybi(self.args[0])
@@ -1632,9 +1719,9 @@ class airybiprime(AiryBase):
             return sqrt(ot) * (b*besseli(-tt, tt*a) + z**2*c*besseli(tt, tt*a))
 
     def _eval_rewrite_as_hyper(self, z, **kwargs):
-        pf1 = z**2 / (2*root(3, 6)*gamma(S(2)/3))
-        pf2 = root(3, 6) / gamma(S(1)/3)
-        return pf1 * hyper([], [S(5)/3], z**3/9) + pf2 * hyper([], [S(1)/3], z**3/9)
+        pf1 = z**2 / (2*root(3, 6)*gamma(Rational(2, 3)))
+        pf2 = root(3, 6) / gamma(Rational(1, 3))
+        return pf1 * hyper([], [Rational(5, 3)], z**3/9) + pf2 * hyper([], [Rational(1, 3)], z**3/9)
 
     def _eval_expand_func(self, **hints):
         arg = self.args[0]
@@ -1660,3 +1747,112 @@ class airybiprime(AiryBase):
                     pf = (d**m * z**(n*m)) / (d * z**n)**m
                     newarg = c * d**m * z**(n*m)
                     return S.Half * (sqrt(3)*(pf - S.One)*airyaiprime(newarg) + (pf + S.One)*airybiprime(newarg))
+
+
+class marcumq(Function):
+    r"""
+    The Marcum Q-function.
+
+    Explanation
+    ===========
+
+    The Marcum Q-function is defined by the meromorphic continuation of
+
+    .. math::
+        Q_m(a, b) = a^{- m + 1} \int_{b}^{\infty} x^{m} e^{- \frac{a^{2}}{2} - \frac{x^{2}}{2}} I_{m - 1}\left(a x\right)\, dx
+
+    Examples
+    ========
+
+    >>> from sympy import marcumq
+    >>> from sympy.abc import m, a, b
+    >>> marcumq(m, a, b)
+    marcumq(m, a, b)
+
+    Special values:
+
+    >>> marcumq(m, 0, b)
+    uppergamma(m, b**2/2)/gamma(m)
+    >>> marcumq(0, 0, 0)
+    0
+    >>> marcumq(0, a, 0)
+    1 - exp(-a**2/2)
+    >>> marcumq(1, a, a)
+    1/2 + exp(-a**2)*besseli(0, a**2)/2
+    >>> marcumq(2, a, a)
+    1/2 + exp(-a**2)*besseli(0, a**2)/2 + exp(-a**2)*besseli(1, a**2)
+
+    Differentiation with respect to $a$ and $b$ is supported:
+
+    >>> from sympy import diff
+    >>> diff(marcumq(m, a, b), a)
+    a*(-marcumq(m, a, b) + marcumq(m + 1, a, b))
+    >>> diff(marcumq(m, a, b), b)
+    -a**(1 - m)*b**m*exp(-a**2/2 - b**2/2)*besseli(m - 1, a*b)
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Marcum_Q-function
+    .. [2] http://mathworld.wolfram.com/MarcumQ-Function.html
+
+    """
+
+    @classmethod
+    def eval(cls, m, a, b):
+        from sympy import exp, uppergamma
+        if a is S.Zero:
+            if m is S.Zero and b is S.Zero:
+                return S.Zero
+            return uppergamma(m, b**2 * S.Half) / gamma(m)
+
+        if m is S.Zero and b is S.Zero:
+            return 1 - 1 / exp(a**2 * S.Half)
+
+        if a == b:
+            if m is S.One:
+                return (1 + exp(-a**2) * besseli(0, a**2))*S.Half
+            if m == 2:
+                return S.Half + S.Half * exp(-a**2) * besseli(0, a**2) + exp(-a**2) * besseli(1, a**2)
+
+        if a.is_zero:
+            if m.is_zero and b.is_zero:
+                return S.Zero
+            return uppergamma(m, b**2*S.Half) / gamma(m)
+
+        if m.is_zero and b.is_zero:
+            return 1 - 1 / exp(a**2*S.Half)
+
+    def fdiff(self, argindex=2):
+        from sympy import exp
+        m, a, b = self.args
+        if argindex == 2:
+            return a * (-marcumq(m, a, b) + marcumq(1+m, a, b))
+        elif argindex == 3:
+            return (-b**m / a**(m-1)) * exp(-(a**2 + b**2)/2) * besseli(m-1, a*b)
+        else:
+            raise ArgumentIndexError(self, argindex)
+
+    def _eval_rewrite_as_Integral(self, m, a, b, **kwargs):
+        from sympy import Integral, exp, Dummy, oo
+        x = kwargs.get('x', Dummy('x'))
+        return a ** (1 - m) * \
+               Integral(x**m * exp(-(x**2 + a**2)/2) * besseli(m-1, a*x), [x, b, oo])
+
+    def _eval_rewrite_as_Sum(self, m, a, b, **kwargs):
+        from sympy import Sum, exp, Dummy, oo
+        k = kwargs.get('k', Dummy('k'))
+        return exp(-(a**2 + b**2) / 2) * Sum((a/b)**k * besseli(k, a*b), [k, 1-m, oo])
+
+    def _eval_rewrite_as_besseli(self, m, a, b, **kwargs):
+        if a == b:
+            from sympy import exp
+            if m == 1:
+                return (1 + exp(-a**2) * besseli(0, a**2)) / 2
+            if m.is_Integer and m >= 2:
+                s = sum([besseli(i, a**2) for i in range(1, m)])
+                return S.Half + exp(-a**2) * besseli(0, a**2) / 2 + exp(-a**2) * s
+
+    def _eval_is_zero(self):
+        if all(arg.is_zero for arg in self.args):
+            return True
