@@ -1,17 +1,15 @@
-from sympy import (KroneckerDelta, diff, Piecewise, Sum, Dummy, factor,
+from sympy import (KroneckerDelta, diff, Sum, Dummy, factor,
                    expand, zeros, gcd_terms, Eq, Symbol)
 
 from sympy.core import S, symbols, Add, Mul, SympifyError, Rational
-from sympy.core.expr import unchanged
-from sympy.core.compatibility import long
-from sympy.functions import transpose, sin, cos, sqrt, cbrt, exp
+from sympy.functions import sin, cos, sqrt, cbrt, exp
 from sympy.simplify import simplify
-from sympy.matrices import (Identity, ImmutableMatrix, Inverse, MatAdd, MatMul,
-        MatPow, Matrix, MatrixExpr, MatrixSymbol, ShapeError, ZeroMatrix,
-        SparseMatrix, Transpose, Adjoint)
-from sympy.matrices.expressions.matexpr import (MatrixElement,
-                                                GenericZeroMatrix, GenericIdentity, OneMatrix)
-from sympy.utilities.pytest import raises, XFAIL
+from sympy.matrices import (ImmutableMatrix, Inverse, MatAdd, MatMul,
+        MatPow, Matrix, MatrixExpr, MatrixSymbol, ShapeError,
+        SparseMatrix, Transpose, Adjoint, NonSquareMatrixError, MatrixSet)
+from sympy.matrices.expressions.matexpr import MatrixElement
+from sympy.matrices.expressions.special import ZeroMatrix, Identity
+from sympy.testing.pytest import raises, XFAIL
 
 
 n, m, l, k, p = symbols('n m l k p', integer=True)
@@ -42,57 +40,6 @@ def test_matrix_symbol_creation():
     raises(ValueError, lambda: MatrixSymbol('A', n, n))
 
 
-def test_zero_matrix_creation():
-    assert unchanged(ZeroMatrix, 2, 2)
-    assert unchanged(ZeroMatrix, 0, 0)
-    raises(ValueError, lambda: ZeroMatrix(-1, 2))
-    raises(ValueError, lambda: ZeroMatrix(2.0, 2))
-    raises(ValueError, lambda: ZeroMatrix(2j, 2))
-    raises(ValueError, lambda: ZeroMatrix(2, -1))
-    raises(ValueError, lambda: ZeroMatrix(2, 2.0))
-    raises(ValueError, lambda: ZeroMatrix(2, 2j))
-
-    n = symbols('n')
-    assert unchanged(ZeroMatrix, n, n)
-    n = symbols('n', integer=False)
-    raises(ValueError, lambda: ZeroMatrix(n, n))
-    n = symbols('n', negative=True)
-    raises(ValueError, lambda: ZeroMatrix(n, n))
-
-
-def test_one_matrix_creation():
-    assert OneMatrix(2, 2)
-    assert OneMatrix(0, 0)
-    raises(ValueError, lambda: OneMatrix(-1, 2))
-    raises(ValueError, lambda: OneMatrix(2.0, 2))
-    raises(ValueError, lambda: OneMatrix(2j, 2))
-    raises(ValueError, lambda: OneMatrix(2, -1))
-    raises(ValueError, lambda: OneMatrix(2, 2.0))
-    raises(ValueError, lambda: OneMatrix(2, 2j))
-
-    n = symbols('n')
-    assert OneMatrix(n, n)
-    n = symbols('n', integer=False)
-    raises(ValueError, lambda: OneMatrix(n, n))
-    n = symbols('n', negative=True)
-    raises(ValueError, lambda: OneMatrix(n, n))
-
-
-def test_identity_matrix_creation():
-    assert Identity(2)
-    assert Identity(0)
-    raises(ValueError, lambda: Identity(-1))
-    raises(ValueError, lambda: Identity(2.0))
-    raises(ValueError, lambda: Identity(2j))
-
-    n = symbols('n')
-    assert Identity(n)
-    n = symbols('n', integer=False)
-    raises(ValueError, lambda: Identity(n))
-    n = symbols('n', negative=True)
-    raises(ValueError, lambda: Identity(n))
-
-
 def test_shape():
     assert A.shape == (n, m)
     assert (A*B).shape == (n, l)
@@ -112,114 +59,14 @@ def test_subs():
     C = MatrixSymbol('C', m, l)
 
     assert A.subs(n, m).shape == (m, m)
-
     assert (A*B).subs(B, C) == A*C
-
     assert (A*B).subs(l, n).is_square
 
+    A = SparseMatrix([[1, 2], [3, 4]])
+    B = Matrix([[1, 2], [3, 4]])
+    C, D = MatrixSymbol('C', 2, 2), MatrixSymbol('D', 2, 2)
 
-def test_ZeroMatrix():
-    A = MatrixSymbol('A', n, m)
-    Z = ZeroMatrix(n, m)
-
-    assert A + Z == A
-    assert A*Z.T == ZeroMatrix(n, n)
-    assert Z*A.T == ZeroMatrix(n, n)
-    assert A - A == ZeroMatrix(*A.shape)
-
-    assert not Z
-
-    assert transpose(Z) == ZeroMatrix(m, n)
-    assert Z.conjugate() == Z
-
-    assert ZeroMatrix(n, n)**0 == Identity(n)
-    with raises(ShapeError):
-        Z**0
-    with raises(ShapeError):
-        Z**2
-
-
-def test_ZeroMatrix_doit():
-    Znn = ZeroMatrix(Add(n, n, evaluate=False), n)
-    assert isinstance(Znn.rows, Add)
-    assert Znn.doit() == ZeroMatrix(2*n, n)
-    assert isinstance(Znn.doit().rows, Mul)
-
-
-def test_OneMatrix():
-    A = MatrixSymbol('A', n, m)
-    a = MatrixSymbol('a', n, 1)
-    U = OneMatrix(n, m)
-
-    assert U.shape == (n, m)
-    assert isinstance(A + U, Add)
-    assert transpose(U) == OneMatrix(m, n)
-    assert U.conjugate() == U
-
-    assert OneMatrix(n, n) ** 0 == Identity(n)
-    with raises(ShapeError):
-        U ** 0
-    with raises(ShapeError):
-        U ** 2
-    with raises(ShapeError):
-        a + U
-
-    U = OneMatrix(n, n)
-    assert U[1, 2] == 1
-
-    U = OneMatrix(2, 3)
-    assert U.as_explicit() == ImmutableMatrix.ones(2, 3)
-
-
-def test_OneMatrix_doit():
-    Unn = OneMatrix(Add(n, n, evaluate=False), n)
-    assert isinstance(Unn.rows, Add)
-    assert Unn.doit() == OneMatrix(2 * n, n)
-    assert isinstance(Unn.doit().rows, Mul)
-
-
-def test_Identity():
-    A = MatrixSymbol('A', n, m)
-    i, j = symbols('i j')
-
-    In = Identity(n)
-    Im = Identity(m)
-
-    assert A*Im == A
-    assert In*A == A
-
-    assert transpose(In) == In
-    assert In.inverse() == In
-    assert In.conjugate() == In
-
-    assert In[i, j] != 0
-    assert Sum(In[i, j], (i, 0, n-1), (j, 0, n-1)).subs(n,3).doit() == 3
-    assert Sum(Sum(In[i, j], (i, 0, n-1)), (j, 0, n-1)).subs(n,3).doit() == 3
-
-    # If range exceeds the limit `(0, n-1)`, do not remove `Piecewise`:
-    expr = Sum(In[i, j], (i, 0, n-1))
-    assert expr.doit() == 1
-    expr = Sum(In[i, j], (i, 0, n-2))
-    assert expr.doit().dummy_eq(
-        Piecewise(
-            (1, (j >= 0) & (j <= n-2)),
-            (0, True)
-        )
-    )
-    expr = Sum(In[i, j], (i, 1, n-1))
-    assert expr.doit().dummy_eq(
-        Piecewise(
-            (1, (j >= 1) & (j <= n-1)),
-            (0, True)
-        )
-    )
-
-
-def test_Identity_doit():
-    Inn = Identity(Add(n, n, evaluate=False))
-    assert isinstance(Inn.rows, Add)
-    assert Inn.doit() == Identity(2*n)
-    assert isinstance(Inn.doit().rows, Mul)
+    assert (C*D).subs({C: A, D: B}) == MatMul(A, B)
 
 
 def test_addition():
@@ -284,7 +131,7 @@ def test_MatPow():
     assert (A**2)**3 == A**6
     assert A**S.Half == sqrt(A)
     assert A**Rational(1, 3) == cbrt(A)
-    raises(ShapeError, lambda: MatrixSymbol('B', 3, 2)**2)
+    raises(NonSquareMatrixError, lambda: MatrixSymbol('B', 3, 2)**2)
 
 
 def test_MatrixSymbol():
@@ -302,7 +149,7 @@ def test_dense_conversion():
 
 
 def test_free_symbols():
-    assert (C*D).free_symbols == set((C, D))
+    assert (C*D).free_symbols == {C, D}
 
 
 def test_zero_matmul():
@@ -341,7 +188,7 @@ def test_indexing():
 def test_single_indexing():
     A = MatrixSymbol('A', 2, 3)
     assert A[1] == A[0, 1]
-    assert A[long(1)] == A[0, 1]
+    assert A[int(1)] == A[0, 1]
     assert A[3] == A[1, 0]
     assert list(A[:2, :2]) == [A[0, 0], A[0, 1], A[1, 0], A[1, 1]]
     raises(IndexError, lambda: A[6])
@@ -493,46 +340,6 @@ def test_issue_7842():
     assert Eq(A, B) == True
 
 
-def test_generic_zero_matrix():
-    z = GenericZeroMatrix()
-    A = MatrixSymbol("A", n, n)
-
-    assert z == z
-    assert z != A
-    assert A != z
-
-    assert z.is_ZeroMatrix
-
-    raises(TypeError, lambda: z.shape)
-    raises(TypeError, lambda: z.rows)
-    raises(TypeError, lambda: z.cols)
-
-    assert MatAdd() == z
-    assert MatAdd(z, A) == MatAdd(A)
-    # Make sure it is hashable
-    hash(z)
-
-
-def test_generic_identity():
-    I = GenericIdentity()
-    A = MatrixSymbol("A", n, n)
-
-    assert I == I
-    assert I != A
-    assert A != I
-
-    assert I.is_Identity
-    assert I**-1 == I
-
-    raises(TypeError, lambda: I.shape)
-    raises(TypeError, lambda: I.rows)
-    raises(TypeError, lambda: I.cols)
-
-    assert MatMul() == I
-    assert MatMul(I, A) == MatMul(A)
-    # Make sure it is hashable
-    hash(I)
-
 def test_MatMul_postprocessor():
     z = zeros(2)
     z1 = ZeroMatrix(2, 2)
@@ -643,3 +450,35 @@ def test_matrixsymbol_from_symbol():
     A_2 = A.subs(2, 3)
     assert A_1.args == A.args
     assert A_2.args[0] == A.args[0]
+
+
+def test_as_explicit():
+    Z = MatrixSymbol('Z', 2, 3)
+    assert Z.as_explicit() == ImmutableMatrix([
+        [Z[0, 0], Z[0, 1], Z[0, 2]],
+        [Z[1, 0], Z[1, 1], Z[1, 2]],
+    ])
+    raises(ValueError, lambda: A.as_explicit())
+
+def test_MatrixSet():
+    M = MatrixSet(2, 2, set=S.Reals)
+    assert M.shape == (2, 2)
+    assert M.set == S.Reals
+    X = Matrix([[1, 2], [3, 4]])
+    assert X in M
+    X = ZeroMatrix(2, 2)
+    assert X in M
+    raises(TypeError, lambda: A in M)
+    raises(TypeError, lambda: 1 in M)
+    M = MatrixSet(n, m, set=S.Reals)
+    assert A in M
+    raises(TypeError, lambda: C in M)
+    raises(TypeError, lambda: X in M)
+    M = MatrixSet(2, 2, set={1, 2, 3})
+    X = Matrix([[1, 2], [3, 4]])
+    Y = Matrix([[1, 2]])
+    assert (X in M) == S.false
+    assert (Y in M) == S.false
+    raises(ValueError, lambda: MatrixSet(2, -2, S.Reals))
+    raises(ValueError, lambda: MatrixSet(2.4, -1, S.Reals))
+    raises(TypeError, lambda: MatrixSet(2, 2, (1, 2, 3)))
